@@ -336,8 +336,7 @@ impl MapxRawMkVs {
         {
             return Err(eg!("version is not on this branch"));
         }
-
-        self.clean_up_version_if_orphaned(version_id).c(d!())
+        Ok(())
     }
 
     #[inline(always)]
@@ -387,14 +386,13 @@ impl MapxRawMkVs {
                 .iter_op(&mut chgset_ops)
                 .c(d!())?;
 
-            vers_hdr.remove(verid).c(d!())?;
-
             self.version_id_to_version_name
                 .remove(verid)
                 .c(d!())
                 .and_then(|vername| {
                     self.version_name_to_version_id.remove(&vername).c(d!())
-                })?;
+                })
+                .and_then(|_| vers_hdr.remove(verid).c(d!()))?;
         }
 
         Ok(())
@@ -696,6 +694,8 @@ impl MapxRawMkVs {
 
     #[inline(always)]
     pub(super) fn prune(&self, reserved_ver_num: Option<usize>) -> Result<()> {
+        self.version_clean_up_global().c(d!())?;
+
         let reserved_ver_num = reserved_ver_num.unwrap_or(RESERVED_VERSION_NUM_DEFAULT);
         if 0 == reserved_ver_num {
             return Err(eg!("reserved version number should NOT be zero"));
@@ -767,24 +767,6 @@ impl MapxRawMkVs {
                 .c(d!())?;
         }
 
-        let valid_vers = self
-            .branch_to_its_versions
-            .iter()
-            .flat_map(|(_, vers)| vers.iter().map(|(ver, _)| ver))
-            .collect::<HashSet<_>>();
-
-        for (ver, chgset) in self
-            .version_to_change_set
-            .iter()
-            .filter(|(ver, _)| !valid_vers.contains(ver))
-        {
-            let mut chgset_ops = |key: &[&[u8]], _: &[u8]| {
-                self.layered_kv.get(key).c(d!())?.remove(&ver).c(d!())?;
-                Ok(())
-            };
-            chgset.iter_op(&mut chgset_ops).c(d!())?;
-        }
-
         Ok(())
     }
 
@@ -798,20 +780,39 @@ impl MapxRawMkVs {
         self.version_name_to_version_id.get(version_name.0)
     }
 
+    // clean up all orphaned versions in the global scope
     #[inline(always)]
-    fn clean_up_version_if_orphaned(&self, version_id: VersionID) -> Result<()> {
-        if self
+    fn version_clean_up_global(&self) -> Result<()> {
+        let valid_vers = self
             .branch_to_its_versions
             .iter()
-            .all(|(_, vers)| !vers.contains_key(&version_id))
+            .flat_map(|(_, vers)| vers.iter().map(|(ver, _)| ver))
+            .collect::<HashSet<_>>();
+
+        for (ver, chgset) in self
+            .version_to_change_set
+            .iter()
+            .filter(|(ver, _)| !valid_vers.contains(ver))
         {
-            if let Some(vername) = self.version_id_to_version_name.remove(&version_id) {
-                self.version_name_to_version_id.remove(&vername).c(d!())?;
-            }
+            let mut chgset_ops = |key: &[&[u8]], _: &[u8]| {
+                self.layered_kv
+                    .get(key)
+                    .c(d!())?
+                    .remove(&ver)
+                    .c(d!())
+                    .map(|_| ())
+            };
+            chgset.iter_op(&mut chgset_ops).c(d!())?;
+
+            self.version_id_to_version_name
+                .remove(&ver)
+                .c(d!())
+                .and_then(|vername| {
+                    self.version_name_to_version_id.remove(&vername).c(d!())
+                })
+                .and_then(|_| self.version_to_change_set.remove(&ver).c(d!()))?;
         }
+
         Ok(())
     }
 }
-
-////////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////////
